#!/usr/bin/python
# coding: utf-8


#This file is part of Block Dude.

#Block Dude is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#Block Dude is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with Block Dude. If not, see <http://www.gnu.org/licenses/>.




 
VERSION = "1.0"
 
try:
        import sys
        import os
        import pygame
        from pygame.locals import *
        import blockDude
        import player
        import level
except ImportError, err:
        print "Error: can't load module. %s" % (err)
        sys.exit(2)


class BlockDude:
    """BlockDude is the main file of the game. It contains everything 
    to load the game, the intro and to play."""    
    
    __images__ = {}
    
    
    def __init__(self):
        os.environ['SDL_VIDEO_CENTERED'] = '1'
        pygame.init()
        screen = pygame.display.set_mode((1024,768))
        pygame.display.set_caption('Block Dude')
        pygame.mouse.set_visible(0)
        self.setImage('background', True)
        self.setImage('block', True)
        self.setImage('brick', True)
        self.setImage('goal')
        self.setImage('starL')
        self.setImage('starR')
        self.setImage('cloud')
        self.setImage('keys', True) # load arrows' keyboard
        self.setImage('root') # load a red root showing which key is used
        
        self.main()
        
        
    def main(self):
        """This method has the main loop of the game"""
        password = ['jbg', 'nfu', 'sol', 'gcr', 'psh', 'cyd', 'xzv', 'skv', 'uti', 'xlq', 'mwa']
        userStop, i = self.intro(password)
        if (i == 0 or i == True):
            i = 1
        while (not userStop and i < 12):
            userStop = self.playLevel(i, password[i-1])
            i += 1
            if (userStop or i == 12):
                userStop, i = self.intro(password)
                if (i == 0 or i == True):                    
                    i = 1
                    
        pygame.quit()        
        sys.exit(0)
        
    
    def setImage(self, name, convert = False):
        """Load image"""
        if (convert):
            self.__images__[name] = pygame.image.load(os.path.join('data', 'images', name + ".png")).convert()
        else:
            self.__images__[name] = pygame.image.load(os.path.join('data', 'images', name + ".png"))
        
        
    def getImage(self, name):
        return self.__images__[name].copy()
        
            
    def intro(self, password):
        """This method load the intro screen and shows an animation to 
        understand how playing the game"""
        background = self.getImage('background')
        for i in range(32):
            background.blit(self.getImage('brick'), (i * 32, 23 * 32))
            
        for i in range(24):
            background.blit(self.getImage('brick'), (0, i * 32))
            background.blit(self.getImage('brick'), (31 * 32, i * 32))


        background.blit(self.getImage('brick'), (4 * 32, 22 * 32))
        background.blit(self.getImage('brick'), (4 * 32, 21 * 32))
        background.blit(self.getImage('goal'), (1 * 32, 22 * 32))
        background.blit(self.getImage('keys'),(16*32+16, 20*32+16))
        
        if pygame.font:
            # Block Dude title
            file = os.path.join('data', 'fonts', "LMS Lance's Letter Blocks.ttf")
            font = pygame.font.Font(file, 128)
            text = font.render("Block", 1, (255, 255, 0))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=120)
            background.blit(text, textpos)
            text = font.render("Dude", 1, (255, 255, 255))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=250)
            background.blit(text, textpos)            
            
            # about
            file = os.path.join('data', 'fonts', "Gilles_Handwriting.ttf")
            font = pygame.font.Font(file, 32)
            text = font.render("Block Dude v 1.0", 1, (0, 0, 0))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=432)
            background.blit(text, textpos)
            text = font.render("By Brandon STERNER", 1, (0, 0, 0))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=464)
            background.blit(text, textpos)
            text = font.render("Coded by Xavier-Emmanuel VINCENT", 1, (0, 0, 0))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=496)
            background.blit(text, textpos)
            text = font.render("blockdude@tuxfamily.org", 1, (0, 0, 0))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=528)
            background.blit(text, textpos)
            
            # instructions
            font = pygame.font.Font(file, 32)
            text = font.render("enter: start game", 1, (0, 0, 0))
            textpos = text.get_rect(left=21*32, centery=632)
            background.blit(text, textpos)
            text = font.render("p      : password", 1, (0, 0, 0))
            textpos = text.get_rect(left=21*32, centery=656)
            background.blit(text, textpos)
            text = font.render("r      : restart level", 1, (0, 0, 0))
            textpos = text.get_rect(left=21*32, centery=688)
            background.blit(text, textpos)
            text = font.render("f      : fullscreen", 1, (0, 0, 0))
            textpos = text.get_rect(left=21*32, centery=720)
            background.blit(text, textpos)
            
        
        # steps is the list for the animation   
        # steps[0] are for the star, steps[1] are for the block
        steps = [[(15,22), (14,22), (13,22), (12,22), (11,22), (10,22), \
        (10,22), (9,22), (8,22), (7,22), (6,22), (6,22), (5,21), (4,20), (3,22), \
        (2,22)],[(9,22), (9,22), (9,22), (9,22), (9,22), (9,22), (10,21), \
        (9,21), (8,21), (7,21), (6,21), (5,22), (5,22), (5,22), (5,22), (5,22)]]
        step = 0
        
        screen = pygame.display.get_surface()
        keyEnter = 0
        userStop = False
        move = 0
        while (not (keyEnter or userStop)):
            e = pygame.event.get()
            for event in e:
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN):
                    if (event.key == K_ESCAPE):
                        userStop = True
                    if (event.key == K_f):
                        pygame.display.toggle_fullscreen()
                    if ((event.key ==K_RETURN) or (event.key == K_KP_ENTER)):
                        keyEnter = True
                    if (event.key == K_p):
                        answer = self.password(screen)
                        if (answer in password):
                            keyEnter = password.index(answer) + 1

            # blit on the background all moving images (star, block, root)
            screen.blit(background, (0,0))
            starPos = steps[0][step]
            screen.blit(self.getImage('starL'), (starPos[0] * 32, starPos[1] * 32))
            blockPos = steps[1][step]
            screen.blit(self.getImage('block'), (blockPos[0] * 32, blockPos[1] * 32))
            if (steps[0][step] == steps[0][step - 1]):
                screen.blit(self.getImage('root'), (17*32+18,21*32+12))
            elif (steps[0][step][1] < steps[0][step - 1][1]):
                screen.blit(self.getImage('root'), (17*32+18,20*32+12))
            else:
                screen.blit(self.getImage('root'), (16*32+14,21*32+12))
            # move var enables to have a slow animation    
            move += 1
            if move >= 10:
                move = 0
                step += 1
            if step >= len(steps[0]):
                step = 0            
                
            pygame.display.flip()
            pygame.time.delay(100)
            
        return userStop, keyEnter
    #
    # End of def start()
    #

    def password(self, screen):
        """Method to get the password"""
        if pygame.font:
            file = os.path.join('data', 'fonts', "Gilles_Handwriting.ttf")
            font = pygame.font.Font(file, 32)
            text = font.render("=>", 1, (0, 0, 0))
            textpos = text.get_rect(left=27*32, centery=656)
            screen.blit(text, textpos)
        keyEnter = False 
        answer = ""
        screenSave = screen.copy()
        while (not keyEnter):
            screen.blit(screenSave, (0,0))
            e = pygame.event.get()
            for event in e:
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN):
                    if ((event.key == K_RETURN) or (event.key == K_KP_ENTER) or (event.key == K_ESCAPE)):
                        keyEnter = True
                    elif ((event.key == K_BACKSPACE) or (event.key == K_DELETE)):
                        if (len(answer)):
                            answer = answer[:-1]
                    else: 
                        answer += pygame.key.name(event.key)
            
            text = font.render(answer, 1, (0, 0, 0))
            textpos = text.get_rect(left=28*32+16, centery=656)
            screen.blit(text, textpos)
            pygame.display.flip()
            pygame.time.delay(100)
            
        return answer
            

    def playLevel(self, number, password):
        """This method is the blackbone of the game. It loads the level and enables to play."""
        lvl = level.Level(number)
        gamer = lvl.getPlayer()
        board = lvl.getBoard()
        center = lvl.getCenter()     
        
        background = self.getImage('background')
        background.blit(self.getImage('cloud'), (32*26,8))
        
        if pygame.font:
            file = os.path.join('data', 'fonts', "LMS Lance's Letter Blocks.ttf")
            font = pygame.font.Font(file, 64)
            text = font.render("Level " + str(number), 1, (255, 255, 0))
            textpos = text.get_rect(centerx=background.get_width()/4, centery=40)
            background.blit(text, textpos)
            text = font.render("Block Dude", 1, (17, 17, 17))
            textpos = text.get_rect(centerx=background.get_width()/2, centery=728)
            background.blit(text, textpos)
            file = os.path.join('data', 'fonts', "Gilles_Handwriting.ttf")
            font = pygame.font.Font(file, 64)
            text = font.render("password:  " + password, 1, (0, 0, 0))
            textpos = text.get_rect(centerx=(32*24), centery=40)
            background.blit(text, textpos)
        
        if gamer.way == "left":
            star = self.getImage('starL')
        else:
            star = self.getImage('starR')
            
        # draws all blocks and the goal
        for key, value in board.items():
            if ((value == 'goal') or (value == 'brick')):
                background.blit(self.getImage(value), ((key[0] + center[0]) * 32, (key[1] + center[1]) * 32))
                
        
        screen = pygame.display.get_surface()
        userStop = False
        while not (gamer.isReaching or userStop):
            e = pygame.event.get()
            for event in e:
                if event.type == pygame.QUIT:
                    sys.exit()
                elif (event.type == pygame.KEYDOWN):
                    if (event.key == K_ESCAPE):
                        userStop = True
                    if event.key == K_f:
                        pygame.display.toggle_fullscreen()
                    if event.key == K_r:
                        # restart the level
                        board = lvl.getBoard()
                        gamer = lvl.getPlayer()
                        if gamer.way == 'right':                    
                            star = self.getImage('starR')
                        else:
                            star = self.getImage('starL')
                    if event.key == pygame.K_LEFT:
                        if gamer.way == 'right':                    
                            star = self.getImage('starL')
                        gamer.moveLR(board, 'left')
                    if event.key == pygame.K_RIGHT:
                        if gamer.way == 'left':
                            star = self.getImage('starR')
                        gamer.moveLR(board, 'right')
                    if event.key == pygame.K_UP:
                        gamer.moveUp(board)
                    if event.key == pygame.K_DOWN:
                        gamer.lift(board)

            # We must to blit each time the player and the blocks as they move
            screen.blit(background, (0,0))
            screen.blit(star, ((gamer.x + center[0]) * 32, (gamer.y + center[1]) * 32))
            for key, value in board.items():
                if value == 'block':
                    screen.blit(self.getImage('block'), ((key[0] + center[0]) * 32, (key[1] + center[1]) * 32))
            
            pygame.display.flip()
            pygame.time.delay(100)
             
        return userStop


"""Launch the class blockDude"""
blockDude.BlockDude()
