# coding: utf-8


#This file is part of Block Dude.

#Block Dude is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#Block Dude is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with Block Dude. If not, see <http://www.gnu.org/licenses/>.


import player


class Level:
    """Level is the class which loads all data about a level."""

    __board__ = None # the board will get all locations of blocks, bricks and goal
    __player__ = None
    __center__ = None # this var enables to center the level on the screen
    
    
    def __init__(self, lvl):
        self.__board__ = {}
        pict = self.getLevel(lvl)
        self.setupLevel(pict)
        
        
    def getLevel(self, lvl):
        """Get the board"""
        if (lvl == 1):
            pict = ["B                  B", \
                    "B                  B", \
                    "B                  B", \
                    "B                  B", \
                    "B                  B", \
                    "B   B       B      B", \
                    "BG  B   B O B O R  B", \
                    "BBBBBBBBBBBBBBBBBBBB"]
        elif (lvl == 2):
            pict = [" B    BB        BB    ", \
                    " B                B   ", \
                    "BB                 B  ", \
                    "BG                  B ", \
                    "BB                   B", \
                    " B           B  O    B", \
                    " B           BO OOL  B", \
                    " BBBBB   BBBBBBBBBBBBB", \
                    "     B  OB            ", \
                    "     BBBBB            "]
        elif (lvl == 3):
            pict = [" B                 ", \
                    " B   BBBBBBBBBBBBB ", \
                    "B B B             B", \
                    "B  B              B", \
                    "B                OB", \
                    "B               OOB", \
                    "B BBB    L   BO BB ", \
                    "B B B    B  BBBBB  ", \
                    "B B BOO BB  B      ", \
                    "BGB BBBBBB BB      ", \
                    "BBB BB   BBB       "]
        elif (lvl == 4):
            pict = ["                  B     ", \
                    "                 B B    ", \
                    "       B        B   B   ", \
                    "      B B      B     B  ", \
                    "   BBB   B    B       B ", \
                    "  B       B  B         B", \
                    " B         BB          B", \
                    " B                    OB", \
                    " B                   OOB", \
                    " B               L   BBB", \
                    "BB    B          B   B  ", \
                    "BG    B O        BBBBB  ", \
                    "BBBBB B O   O  BBB      ", \
                    "    B B O B BO B        ", \
                    "    B BBBBBBBBBB        ", \
                    "    BBB                 "]
        elif (lvl == 5):
            pict = ["     BBB    BBBBBBBBB ", \
                    " BBBB   BBBB         B", \
                    "B                    B", \
                    "B                    B", \
                    "B                    B", \
                    "B     B              B", \
                    "B     B              B", \
                    "B     BOOOO          B", \
                    "BG   BBBBBBBL        B", \
                    "BB BBB     BB B     OB", \
                    " B B        B BB   OOB", \
                    " B B        B BB  OOOB", \
                    " BBB        B BBBBBBBB", \
                    "            BBB       "]
        elif (lvl == 6):
            pict = [" BBB             BBBB", \
                    " B  BBBBBBBBBBBBB   B", \
                    "BB                  B", \
                    "BG                  B", \
                    "BB                  B", \
                    " B                OOB", \
                    " BOO        B  O  BBB", \
                    " BOOO       BLOOO B  ", \
                    " BOOOO      BBBBB B  ", \
                    " BBBBB    BBB   BBB  ", \
                    "     B   OB          ", \
                    "     BB BBB          ", \
                    "      BBB            "]
        elif (lvl == 7):
            pict = ["  B   BBBBB   BB   BBB  ", \
                    " B B B     B B  B B   B ", \
                    " B  BB      BB   BB    B", \
                    " B   B       B    B    B", \
                    " B                    OB", \
                    " B                    OB", \
                    "BB                   OOB", \
                    "BG   O               BBB", \
                    "BB   B O     B    BB B  ", \
                    " B   B O    BB O LBBBB  ", \
                    " BB  B OOO  BB OOOB     ", \
                    "  B  BBBBBB BBBBBBB     ", \
                    "  BB B    BBB           ", \
                    "   BBB                  "]
        elif (lvl == 8):
            pict = [" BBB       BBBB   BBBBBBB  ", \
                    "B   B     B    B B       B ", \
                    "B    B   B     BB         B", \
                    "BO    BBB    B B     BBB  B", \
                    "BOO         BB      BB B  B", \
                    "BBBB       BB          BG B", \
                    "   BB            BB    BB B", \
                    "  B    O B      B  B      B", \
                    "  B    OB B    B   B      B", \
                    " B   BBB   B    B  B     OB", \
                    " B      B B      BB     OOB", \
                    "B        B           BBBBBB", \
                    "B            O            B", \
                    "B    O      BBB          OB", \
                    "B   BBB                 OOB", \
                    "B        O       O  L  OOOB", \
                    "BBBBBBBBBBBBBBBBBBBBBBBBBBB"]
        elif (lvl == 9):
            pict = ["        BBB         ", \
                    "       B   B        ", \
                    "      B     B  BBBBB", \
                    "     B       BB    B", \
                    "    B     O        B", \
                    "   B      OO      OB", \
                    "  B       BBB    OOB", \
                    " B            L BBBB", \
                    "B             O    B", \
                    "BG           BBB   B", \
                    "BB    BB   B      OB", \
                    " B    BBO  BB   BBBB", \
                    " B    BBBBBBB  BB   ", \
                    " BBB  B     B BB    ", \
                    "   B BB     BBB     ", \
                    "   BBB              "]
        elif (lvl == 10):
            pict = ["   BBBBBBBBBBBBBBBBBBBBB   ", \
                    " BB           B         B  ", \
                    "BBBBO       OOBO   OOO OBB ", \
                    "B  BB  B   BBBBB  OBBB BB B", \
                    "B   B  BB        BBB BBB  B", \
                    "B   BB  BBOOOO            B", \
                    "BG       BBBBBBB          B", \
                    "BB        B   BBB        BB", \
                    " B     O   B B  BB        B", \
                    " B     B    B    BB       B", \
                    " BBBB  BB             BBBBB", \
                    "   BBBBB      L           B", \
                    "   B          B           B", \
                    "   B         BB    BBBBBBBB", \
                    "   B        BB           B ", \
                    "   B          O         OB ", \
                    "   BO    BBBBBBBBBBB   OOB ", \
                    "   BOO  BB         BB OOOB ", \
                    "   BBBBBB           BBBBBB "]
        elif (lvl == 11):
            pict = ["BBBBBBBBBBBBBBBBBBBBBBBBBBBBB", \
                    "B  B   B                    B", \
                    "B     OBOO            BBBBB B", \
                    "BO   BBB OBB     O  BB  G B B", \
                    "BOO    BBB   L  O       B B B", \
                    "BBB  OOB     B O          B B", \
                    "B   BBBB      B  BBB   BBB  B", \
                    "BO            B B      B  O B", \
                    "BOO       BBB B BO    B  BBBB", \
                    "BBBB O   BBB  B BBO  B O B  B", \
                    "B           O BBB  OB   B   B", \
                    "B   O     OO B   BBBB       B", \
                    "B    BBBBBBBBB        BBBBB B", \
                    "B              O   OBB    B B", \
                    "BBBB           O   B    OOB B", \
                    "BOBB   B    B          BBBB B", \
                    "BBOBBB B    B   OOO O       B", \
                    "BOBOBOBB    B        OOO    B", \
                    "BBBBBBBBBBBBBBBBBBBBBBBBBBBBB"]
            
        return pict
        
        
    
    def setupLevel(self, lvl):
        """Extract all data of the level"""
        self.__center__=((32.0 - len(lvl[0]))/2, (24.0 - len(lvl))/2)
        for y in range(len(lvl)):
            x = 0
            for j in lvl[y]:
                if j == 'B':
                    self.__board__[(x,y)] = 'brick'
                elif j == 'G':
                    self.__board__[(x,y)] = 'goal'
                elif j == 'O':
                    self.__board__[(x,y)] = 'block'
                elif j == 'L':
                    self.__player__ = player.Player(x, y, 'left')
                elif j == 'R':
                    self.__player__ = player.Player(x, y, 'right')
                
                x += 1
                
                
    def getBoard(self):
        """return the board data"""
        return self.__board__.copy()
        
    def getPlayer(self):
        """return the player data"""
        p = self.__player__
        return player.Player(p.x, p.y, p.way)
        
    def getCenter(self):
        """return the center data"""
        return self.__center__
