# coding: utf-8


#This file is part of Block Dude.

#Block Dude is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#Block Dude is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with Block Dude. If not, see <http://www.gnu.org/licenses/>.


class Player:
    """Player gets all data about the player and enables to move the player"""
    
    isReaching = False # if the player reachs the door
    
    
    def __init__(self, x, y, way = 'left', carry = False):
        self.x = x
        self.y = y
        self.way = way      # going to right or left
        self.carry = carry  # carrying a block
        
     
     
    def moveLR(self, board, way):
        """The left/right arrow has been pressed."""
        if (way != self.way):
            self.way = way
            
        if (self.way == 'right'):
            x = self.x + 1
        else:
            x = self.x - 1
            
        # check if the goal will be reach
        self.isGoal(board, x, self.y)
        
        # check there is nothing at the right/left    
        if board.get((x, self.y), 0) == 0:
            if self.carry:
                # check there is nothing at the right/left for the block
                if board.get((x, self.y - 1), 0) == 0:
                   del board[(self.x, self.y - 1)]
                   board[(x, self.y - 1)] = 'block'
                else:
                    # there is something, the block falls
                    self.fallBlock(board, self.x, self.y -1)
                    self.carry = False
                    
            self.x = x
            self.fall(board)        
    
        
    def moveUp(self, board):
        """The up arrow has been pressed"""
        if (self.way == 'right'):
            x = self.x + 1
        else:
            x = self.x - 1
         
        # check if the goal will be reach    
        self.isGoal(board, x, self.y - 1)
        # check if there is nothing up left/right
        if (board.get((x, self.y), 0) != 0 and 
        board.get((x, self.y - 1), 0) == 0):            
            if self.carry:
                if board.get((x, self.y - 2), 0) == 0:
                   del board[(self.x, self.y - 1)]
                   board[(x, self.y - 2)] = 'block'
                else:
                    self.fallBlock(board, self.x, self.y -1)
                    self.carry = False
                    
            self.x = x
            self.y = self.y - 1 # the top is 0 and the bottom is 31
        
    
    def lift(self, board):
        """The down arrow has been pressed."""
        if (self.way == 'right'):
            x = self.x + 1
        else:
            x = self.x - 1
        
        # check everything is free to carry
        if ((self.carry == False) and 
        (board.get((x, self.y), 0) == 'block') and 
        (board.get((self.x, self.y - 1), 0) == 0) and
        (board.get((x, self.y - 1), 0) == 0)):
            del board[(x, self.y)]
            board[(self.x, self.y - 1)] = 'block'
            self.carry = True
        elif ((self.carry == True) and board.get((x, self.y - 1), 0) == 0):
            del board[(self.x, self.y - 1)]
            i = 0
            # when we lift down, the block can fall more than one brick
            while board.get((x, self.y + i), 0) == 0:
                i += 1
            board[(x, self.y + i - 1)] = 'block'
            self.carry = False
        
        
    def fall(self, board):
        """The player can fall when he moves left/right"""
        while board.get((self.x, self.y + 1), 0) == 0:
            if self.carry:
                del board[(self.x, self.y - 1)]
                board[(self.x, self.y)] = 'block'
            self.y = self.y + 1
            
        self.isGoal(board, self.x, self.y + 1)
        
    
    def fallBlock(self, board, x, y):
        """When the player moves and the block cannot, it falls"""
        del board[(x, y)]
        while board.get((x, y + 1), 0) == 0:
            y +=  1
        board[(x,y)] = 'block'    
            
    def isGoal(self, board, x, y):
        """Check if the player reaches the goal"""
        if board.get((x, y), 0) == 'goal':
            self.isReaching = True
